@@ -17,20 +17,25 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
+import doctest
 import os
 import unittest
 import sys
-import pkg_resources
-from fnmatch import fnmatch
+
+try:
+    from babel import Locale
+except ImportError:
+    Locale = None
 
 from trac.config import Configuration
-from trac.core import Component, ComponentManager, ExtensionPoint
+from trac.core import Component, ComponentManager
 from trac.env import Environment
 from trac.db.api import _parse_db_str, DatabaseManager
 from trac.db.sqlite_backend import SQLiteConnection
 import trac.db.postgres_backend
 import trac.db.mysql_backend
 from trac.ticket.default_workflow import load_workflow_config_snippet
+from trac.util import translation
 
 
 def Mock(bases=(), *initargs, **kw):
@@ -85,7 +90,7 @@ def Mock(bases=(), *initargs, **kw):
         bases = (bases,)
     cls = type('Mock', bases, {})
     mock = cls(*initargs)
-    for k,v in kw.items():
+    for k, v in kw.items():
         setattr(mock, k, v)
     return mock
 
@@ -93,14 +98,18 @@ def Mock(bases=(), *initargs, **kw):
 class MockPerm(object):
     """Fake permission class. Necessary as Mock can not be used with operator
     overloading."""
-    def has_permission(self, x):
+
+    username = ''
+    
+    def has_permission(self, action, realm_or_resource=None, id=False,
+                       version=False):
         return True
     __contains__ = has_permission
 
-    def __call__(self, *a, **kw):
+    def __call__(self, realm_or_resource, id=False, version=False):
         return self
 
-    def require(self, *a, **kw):
+    def require(self, action, realm_or_resource=None, id=False, version=False):
         pass
     assert_permission = require
 
@@ -126,16 +135,19 @@ class TestSetup(unittest.TestSuite):
             for test in self._tests:
                 if hasattr(test, 'setFixture'):
                     test.setFixture(self.fixture)
-        for test in self._tests:    # Content of unittest.TestSuite.run()
-            if result.shouldStop:   # copied here for Python 2.3 compatibility
-                break
-            test(result)
+        unittest.TestSuite.run(self, result)
         self.tearDown()
         return result
 
-    def __call__(self, *args, **kwds):      # Python 2.3 compatibility
-        return self.run(*args, **kwds)
-
+    def _wrapped_run(self, *args, **kwargs):
+        "Python 2.7 / unittest2 compatibility - there must be a better way..."
+        self.setUp()
+        if hasattr(self, 'fixture'):
+            for test in self._tests:
+                if hasattr(test, 'setFixture'):
+                    test.setFixture(self.fixture)
+        unittest.TestSuite._wrapped_run(self, *args, **kwargs)
+        self.tearDown()
 
 class TestCaseSetup(unittest.TestCase):
     def setFixture(self, fixture):
@@ -165,9 +177,16 @@ def reset_postgres_db(db, db_prop):
     if dbname:
         cursor = db.cursor()
         # reset sequences
-        cursor.execute('SELECT sequence_name '
-                       'FROM information_schema.sequences '
-                       'WHERE sequence_schema=%s', (dbname,))
+        # information_schema.sequences view is available in PostgreSQL 8.2+
+        # however Trac supports PostgreSQL 8.0+, uses pg_get_serial_sequence()
+        cursor.execute("""
+            SELECT sequence_name FROM (
+                SELECT pg_get_serial_sequence(%s||table_name, column_name)
+                       AS sequence_name
+                FROM information_schema.columns
+                WHERE table_schema=%s) AS tab
+            WHERE sequence_name IS NOT NULL""",
+            (dbname + '.', dbname))
         for seq in cursor.fetchall():
             cursor.execute('ALTER SEQUENCE %s RESTART WITH 1' % seq)
         # clear tables
@@ -226,14 +245,14 @@ class EnvironmentStub(Environment):
     def __init__(self, default_data=False, enable=None):
         """Construct a new Environment stub object.
 
-        default_data: If True, populate the database with some defaults.
-        enable: A list of component classes or name globs to activate in the
-                stub environment.
+        :param default_data: If True, populate the database with some
+                             defaults.
+        :param enable: A list of component classes or name globs to
+                       activate in the stub environment.
         """
         ComponentManager.__init__(self)
         Component.__init__(self)
-        self.enabled_components = enable or ['trac.*']
-        self.systeminfo = [('Python', sys.version)]
+        self.systeminfo = []
 
         import trac
         self.path = os.path.dirname(trac.__file__)
@@ -248,14 +267,20 @@ class EnvironmentStub(Environment):
         load_workflow_config_snippet(self.config, 'basic-workflow.ini')
         self.config.set('logging', 'log_level', 'DEBUG')
         self.config.set('logging', 'log_type', 'stderr')
+        if enable is not None:
+            self.config.set('components', 'trac.*', 'disabled')
+        for name_or_class in enable or ():
+            config_key = self._component_name(name_or_class)
+            self.config.set('components', config_key, 'enabled')
 
         # -- logging
-        from trac.log import logger_factory
-        self.log = logger_factory('test')
+        from trac.log import logger_handler_factory
+        self.log, self._log_handler = logger_handler_factory('test')
 
         # -- database
         self.dburi = get_dburi()
         if self.dburi.startswith('sqlite'):
+            self.config.set('trac', 'database', 'sqlite::memory:')
             self.db = InMemoryDatabase()
 
         if default_data:
@@ -266,17 +291,12 @@ class EnvironmentStub(Environment):
         self.abs_href = Href('http://example.org/trac.cgi')
 
         self.known_users = []
-
-    def is_component_enabled(self, cls):
-        for component in self.enabled_components:
-            if component is cls:
-                return True
-            if isinstance(component, basestring) and \
-                fnmatch(cls.__module__ + '.' + cls.__name__, component):
-                return True
-        return False
-
-    def get_db_cnx(self):
+        translation.activate(Locale and Locale('en', 'US'))
+        
+    def get_read_db(self):
+        return self.get_db_cnx()
+    
+    def get_db_cnx(self, destroying=False):
         if self.db:
             return self.db # in-memory SQLite
 
@@ -289,7 +309,8 @@ class EnvironmentStub(Environment):
         if not dbenv:
             dbenv = EnvironmentStub.dbenv = EnvironmentStub()
             dbenv.config.set('trac', 'database', self.dburi)
-            self.reset_db() # make sure we get rid of garbage from previous run
+            if not destroying:
+                self.reset_db() # make sure we get rid of previous garbage
         return DatabaseManager(dbenv).get_connection()
 
     def reset_db(self, default_data=None):
@@ -327,7 +348,11 @@ class EnvironmentStub(Environment):
 
             if not tables:
                 del db
-                DatabaseManager(EnvironmentStub.dbenv).init_db()
+                dm = DatabaseManager(EnvironmentStub.dbenv)
+                dm.init_db()
+                # we need to make sure the next get_db_cnx() will re-create 
+                # a new connection aware of the new data model - see #8518.
+                dm.shutdown() 
 
         db = self.get_db_cnx()
         cursor = db.cursor()
@@ -347,7 +372,7 @@ class EnvironmentStub(Environment):
         if not (scheme and db_prop):
             scheme, db_prop = _parse_db_str(self.dburi)
 
-        db = self.get_db_cnx()
+        db = self.get_db_cnx(destroying=True)
         cursor = db.cursor()
         try:
             if scheme == 'postgres' and db.schema:
@@ -362,9 +387,16 @@ class EnvironmentStub(Environment):
                 for t in tables:
                     cursor.execute('DROP TABLE IF EXISTS `%s`' % t)
             db.commit()
-        except Exception, e:
+        except Exception:
             db.rollback()
 
+    # overriden
+
+    def is_component_enabled(self, cls):
+        if self._component_name(cls).startswith('__main__.'):
+            return True
+        return Environment.is_component_enabled(self, cls)
+
     def get_known_users(self, cnx=None):
         return self.known_users
 
@@ -374,7 +406,6 @@ def locate(fn):
 
     Returns the fully-qualified path, or None.
     """
-    import os
     exec_suffix = os.name == 'nt' and '.exe' or ''
     
     for p in ["."] + os.environ['PATH'].split(os.pathsep):
@@ -397,6 +428,8 @@ def suite():
     import trac.versioncontrol.web_ui.tests
     import trac.web.tests
     import trac.wiki.tests
+    import tracopt.mimeview.tests
+    import tracopt.perm.tests
 
     suite = unittest.TestSuite()
     suite.addTest(trac.tests.basicSuite())
@@ -411,19 +444,13 @@ def suite():
     suite.addTest(trac.versioncontrol.web_ui.tests.suite())
     suite.addTest(trac.web.tests.suite())
     suite.addTest(trac.wiki.tests.suite())
+    suite.addTest(tracopt.mimeview.tests.suite())
+    suite.addTest(tracopt.perm.tests.suite())
+    suite.addTest(doctest.DocTestSuite(sys.modules[__name__]))
 
     return suite
 
 if __name__ == '__main__':
-    import doctest, sys
-    doctest.testmod(sys.modules[__name__])
-
-    # Clean up after doctest or spambayes gets unhappy
-    try:
-        del __builtins__._
-    except NameError:
-        pass
-
     #FIXME: this is a bit inelegant
     if '--skip-functional-tests' in sys.argv:
         sys.argv.remove('--skip-functional-tests')
