@@ -2,7 +2,7 @@
 #
 # Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
-# Copyright (C) 2006 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
 # All rights reserved.
 #
@@ -29,10 +29,10 @@ from trac.mimeview import Context
 from trac.perm import IPermissionRequestor
 from trac.resource import Resource, ResourceNotFound
 from trac.ticket.api import TicketSystem
-from trac.util import as_int
+from trac.util import as_int, content_disposition
 from trac.util.datefmt import format_datetime, format_time, from_utimestamp
 from trac.util.presentation import Paginator
-from trac.util.text import to_unicode
+from trac.util.text import to_unicode, quote_query_string
 from trac.util.translation import _, tag_
 from trac.web.api import IRequestHandler, RequestDone
 from trac.web.chrome import add_ctxtnav, add_link, add_notice, add_script, \
@@ -324,7 +324,7 @@ class ReportModule(Component):
                 if query[-1] != '?':
                     query += '&'
                 query += report_id
-            req.redirect(req.href.query() + query)
+            req.redirect(req.href.query() + quote_query_string(query))
         elif query.startswith('query:'):
             try:
                 from trac.ticket.query import Query, QuerySyntaxError
@@ -489,6 +489,8 @@ class ReportModule(Component):
             cell_groups = []
             row = {'cell_groups': cell_groups}
             realm = 'ticket'
+            parent_realm = ''
+            parent_id = ''
             email_cells = []
             for header_group in header_groups:
                 cell_group = []
@@ -515,9 +517,17 @@ class ReportModule(Component):
                         email_cells.append(cell)
                     elif col == 'realm':
                         realm = value
+                    elif col == 'parent_realm':
+                        parent_realm = value
+                    elif col == 'parent_id':
+                        parent_id = value
                     cell_group.append(cell)
                 cell_groups.append(cell_group)
-            resource = Resource(realm, row.get('id'))
+            if parent_realm:
+                resource = Resource(realm, row.get('id'),
+                                    parent=Resource(parent_realm, parent_id))
+            else:
+                resource = Resource(realm, row.get('id'))
             # FIXME: for now, we still need to hardcode the realm in the action
             if resource.realm.upper()+'_VIEW' not in req.perm(resource):
                 continue
@@ -538,7 +548,7 @@ class ReportModule(Component):
         data.update({'header_groups': header_groups,
                      'row_groups': row_groups,
                      'numrows': numrows,
-                     'sorting_enabled': len(row_groups) == 1})
+                     'sorting_enabled': '__group__' not in cols})
 
         if format == 'rss':
             data['email_map'] = chrome.get_email_map()
@@ -628,9 +638,9 @@ class ReportModule(Component):
             sort_col = req.args.get('sort', '')
             self.log.debug("Columns %r, Sort column %s" % (cols, sort_col))
             order_cols = []
+            if '__group__' in cols:
+                sort_col = '' # sorting is disabled (#15030)
             if sort_col:
-                if '__group__' in cols:
-                    order_cols.append('__group__')
                 if sort_col in cols:
                     order_cols.append(sort_col)
                 else:
@@ -753,7 +763,8 @@ class ReportModule(Component):
         req.send_header('Content-Type', mimetype + ';charset=utf-8')
         req.send_header('Content-Length', len(data))
         if filename:
-            req.send_header('Content-Disposition', 'filename=' + filename)
+            req.send_header('Content-Disposition',
+                            content_disposition('attachment', filename))
         req.end_headers()
         req.write(data)
         raise RequestDone
@@ -774,7 +785,8 @@ class ReportModule(Component):
         req.send_header('Content-Length', len(data))
         if id:
             req.send_header('Content-Disposition',
-                            'filename=report_%s.sql' % id)
+                            content_disposition('attachment',
+                                                'report_%s.sql' % id))
         req.end_headers()
         req.write(data)
         raise RequestDone
@@ -785,7 +797,8 @@ class ReportModule(Component):
         yield ('report', self._format_link)
 
     def get_wiki_syntax(self):
-        yield (r"!?\{(?P<it_report>%s\s*)\d+\}" % WikiParser.INTERTRAC_SCHEME,
+        yield (r"!?\{(?P<it_report>%s\s*)[0-9]+\}" % \
+                                                WikiParser.INTERTRAC_SCHEME,
                lambda x, y, z: self._format_link(x, 'report', y[1:-1], y, z))
 
     def _format_link(self, formatter, ns, target, label, fullmatch=None):
