@@ -2,7 +2,7 @@
 #
 # Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
-# Copyright (C) 2005-2007 Christian Boos <cboos@neuf.fr>
+# Copyright (C) 2005-2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -426,7 +426,7 @@ class BrowserModule(Component):
         # Links for contextual navigation
         if node:
             if node.isfile:
-                prev_rev = repos.previous_rev(rev=node.rev,
+                prev_rev = repos.previous_rev(rev=node.created_rev,
                                               path=node.created_path)
                 if prev_rev:
                     href = req.href.browser(reponame,
@@ -436,7 +436,7 @@ class BrowserModule(Component):
                 if rev is not None:
                     add_link(req, 'up', req.href.browser(reponame,
                                                          node.created_path))
-                next_rev = repos.next_rev(rev=node.rev,
+                next_rev = repos.next_rev(rev=node.created_rev,
                                           path=node.created_path)
                 if next_rev:
                     href = req.href.browser(reponame, node.created_path,
@@ -450,17 +450,18 @@ class BrowserModule(Component):
                     add_link(req, 'up', path_links[-2]['href'],
                              _('Parent directory'))
                 add_ctxtnav(req, tag.a(_('Last Change'), 
-                            href=req.href.changeset(node.rev, reponame,
+                            href=req.href.changeset(node.created_rev, reponame,
                                                     node.created_path)))
             if node.isfile:
-                if data['file']['annotate']:
+                annotate = data['file']['annotate']
+                if annotate:
                     add_ctxtnav(req, _('Normal'), 
                                 title=_('View file without annotations'), 
                                 href=req.href.browser(reponame,
                                                       node.created_path, 
                                                       rev=rev))
-                else:
-                    add_ctxtnav(req, _('Annotate'), 
+                if annotate != 'blame':
+                    add_ctxtnav(req, _('Blame'), 
                                 title=_('Annotate each line with the last '
                                         'changed revision '
                                         '(this can be time consuming...)'), 
@@ -541,14 +542,17 @@ class BrowserModule(Component):
 
         # Entries metadata
         class entry(object):
-            __slots__ = 'name rev kind isdir path content_length'.split()
+            __slots__ = ('name', 'rev', 'created_rev', 'kind', 'isdir', 'path',
+                         'content_length')
+
             def __init__(self, node):
                 for f in entry.__slots__:
                     setattr(self, f, getattr(node, f))
                 
         entries = [entry(n) for n in node.get_entries()
                    if n.can_view(req.perm)]
-        changes = get_changes(repos, [i.rev for i in entries], self.log)
+        changes = get_changes(repos, [i.created_rev for i in entries],
+                              self.log)
 
         if rev:
             newest = repos.get_changeset(rev).date
@@ -571,7 +575,7 @@ class BrowserModule(Component):
         # Ordering of entries
         if order == 'date':
             def file_order(a):
-                return (changes[a.rev].date,
+                return (changes[a.created_rev].date,
                         embedded_numbers(a.name.lower()))
         elif order == 'size':
             def file_order(a):
@@ -579,7 +583,7 @@ class BrowserModule(Component):
                         embedded_numbers(a.name.lower()))
         elif order == 'author':
             def file_order(a):
-                return (changes[a.rev].author.lower(),
+                return (changes[a.created_rev].author.lower(),
                         embedded_numbers(a.name.lower()))
         else:
             def file_order(a):
@@ -653,7 +657,7 @@ class BrowserModule(Component):
         else:
             # The changeset corresponding to the last change on `node` 
             # is more interesting than the `rev` changeset.
-            changeset = repos.get_changeset(node.rev)
+            changeset = repos.get_changeset(node.created_rev)
 
             # add ''Plain Text'' alternate link if needed
             if not is_binary(chunk) and mime_type != 'text/plain':
@@ -676,21 +680,20 @@ class BrowserModule(Component):
             add_stylesheet(req, 'common/css/code.css')
 
             annotations = ['lineno']
-            force_source = False
-            if 'annotate' in req.args:
-                force_source = True
-                annotations.insert(0, req.args['annotate'])
+            annotate = req.args.get('annotate')
+            if annotate:
+                annotations.insert(0, annotate)
             preview_data = mimeview.preview_data(context, node.get_content(),
                                                  node.get_content_length(),
                                                  mime_type, node.created_path,
                                                  raw_href,
                                                  annotations=annotations,
-                                                 force_source=force_source)
+                                                 force_source=bool(annotate))
             return {
                 'changeset': changeset,
                 'size': node.content_length,
                 'preview': preview_data,
-                'annotate': force_source,
+                'annotate': annotate,
                 }
 
     # public methods
